package cn.dankal.share.beanpostprocessor;

import org.springframework.beans.BeansException;
import org.springframework.beans.factory.config.BeanPostProcessor;

/**
 * {@link BeanPostProcessor}
 *
 * @author ZGH.MercyModest
 * @version V1.0.0
 * @create 2020/11/25
 */
public class BaseBeanPostProcessor implements BeanPostProcessor {

    /**
     * 执行 bean 的 初始化方法执行之前执行
     *
     * @param bean     当 bean实例对象
     * @param beanName 当前 bean 在 IOC  容器中 的名称
     * @return 当前 bean 实例
     * @throws BeansException
     */
    @Override
    public Object postProcessBeforeInitialization(Object bean, String beanName) throws BeansException {
        System.out.println("BaseBeanPostProcessor ==> postProcessBeforeInitialization: " + beanName);
        return bean;
    }

    /**
     * 执行 bean 的 初始化方法执行之后执行
     *
     * @param bean     当 bean实例对象
     * @param beanName 当前 bean 在 IOC  容器中 的名称
     * @return 当前 bean 实例
     * @throws BeansException
     */
    @Override
    public Object postProcessAfterInitialization(Object bean, String beanName) throws BeansException {
        System.out.println("BaseBeanPostProcessor ==> postProcessAfterInitialization: " + beanName);
        return bean;
    }
}
