package cn.dankal.share.importbeandefinition;

import cn.dankal.share.entity.Study;
import org.springframework.beans.factory.support.AbstractBeanDefinition;
import org.springframework.beans.factory.support.BeanDefinitionBuilder;
import org.springframework.beans.factory.support.BeanDefinitionRegistry;
import org.springframework.context.annotation.ImportBeanDefinitionRegistrar;
import org.springframework.core.type.AnnotationMetadata;

/**
 * {@link ImportBeanDefinitionRegistrar}
 * StudyRegistrar
 *
 * @author ZGH.MercyModest
 * @version V1.0.0
 * @create 2020/11/25
 */
public class StudyRegistrar implements ImportBeanDefinitionRegistrar {


    /**
     * register bean definitions
     *
     * @param importingClassMetadata AnnotationMetadata
     * @param registry               BeanDefinitionRegistry
     */
    @Override
    public void registerBeanDefinitions(AnnotationMetadata importingClassMetadata, BeanDefinitionRegistry registry) {
        // 建议 通过 BeanDefinitionBuilder 构建 BeanDefinition  因为TA会帮对一些成员属性进行默认初数化
        // 而我们直接 new 的话,则不会
        BeanDefinitionBuilder beanDefinitionBuilder = BeanDefinitionBuilder.genericBeanDefinition();
        AbstractBeanDefinition studyBeanDefinition = beanDefinitionBuilder.getBeanDefinition();
        studyBeanDefinition.setBeanClass(Study.class);
        registry.registerBeanDefinition("study-mercymodest", studyBeanDefinition);

        // 设置作用域
        //studyBeanDefinition.setScope();

        // 初始化这个 bean  之前,需要先初始那些bean
        //studyBeanDefinition.setDependsOn();

        // 如果是单实例 bean 是否懒加载
        //studyBeanDefinition.setLazyInit();
    }
}
