package cn.dankal.share.test;

import cn.dankal.share.beanpostprocessor.BeanPostProcessorConfig;
import cn.dankal.share.condition.RegisterConditionConfig;
import cn.dankal.share.factory.FactoryBeanConfig;
import cn.dankal.share.importbeandefinition.BeanRegistrarConfig;
import cn.dankal.share.imports.ImportRegisterBeanConfiguration;
import cn.dankal.share.lifecycle.BeanLifecycleConfig;
import cn.dankal.share.register.RegisterBean;
import cn.dankal.share.register.RegisterBeanWithScope;
import cn.dankal.share.scope.StudyMapper;
import cn.dankal.share.scope.SubjectMapper;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.context.ConfigurableApplicationContext;
import org.springframework.context.annotation.AnnotationConfigApplicationContext;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit4.SpringJUnit4ClassRunner;

import java.util.Arrays;

/**
 * {@link ContextConfiguration} {@link RunWith}  {@link SpringJUnit4ClassRunner}
 * SpringIocMainTest
 *
 * @author ZGH.MercyModest
 * @version V1.0.0
 * @create 2020/11/23
 */
@RunWith(SpringJUnit4ClassRunner.class)
@ContextConfiguration(locations = {"classpath:spring-application.xml"})
public class SpringIocMainTest {

    /**
     * the  applicationContext by  xml class path config file
     */
    @Autowired
    private ApplicationContext applicationContext;


    /**
     * 基于 XML
     */
    @Test
    public void testIocBeanDefinitionListByXml() {
        Arrays.stream(applicationContext.getBeanDefinitionNames())
                .filter(beanName -> !beanName.contains("org.springframework"))
                .forEach(System.out::println);
    }


    /**
     * 基于配置类 (注解)
     */
    @Test
    public void testIocBeanDefinitionListByConfiguration() {
        ApplicationContext applicationContext = new AnnotationConfigApplicationContext(RegisterBean.class);
        Arrays.stream(applicationContext.getBeanDefinitionNames())
                .filter(beanName -> !beanName.contains("org.springframework"))
                .forEach(System.out::println);
    }


    /**
     * 测试 Spring Bean 的作用域
     */
    @Test
    public void testIocBeanWithScope() {
        ApplicationContext applicationContext = new AnnotationConfigApplicationContext(RegisterBeanWithScope.class);
        System.out.println("=============== StudyMapper ===============");
        System.out.println(applicationContext.getBeansOfType(StudyMapper.class).toString());
        System.out.println(applicationContext.getBeansOfType(StudyMapper.class).toString());
        System.out.println(applicationContext.getBeansOfType(StudyMapper.class).toString());
        System.out.println("=============== SubjectMapper ===============");
        System.out.println(applicationContext.getBeansOfType(SubjectMapper.class).toString());
        System.out.println(applicationContext.getBeansOfType(SubjectMapper.class).toString());
        System.out.println(applicationContext.getBeansOfType(SubjectMapper.class).toString());
    }


    /**
     * Spring 单实例 Bean 的懒加载
     */
    @Test
    public void testSingletonBeanLazy() {
        // 创建 Spring IOC 容器对象. 即完成Spring IOC 容器的初始化
        AnnotationConfigApplicationContext applicationContext = new AnnotationConfigApplicationContext(RegisterBeanWithScope.class);
        System.out.println("the applicationContext is finished initialize");
        System.out.println("=============== SubjectMapper ===============");
        System.out.println(applicationContext.getBeansOfType(SubjectMapper.class).toString());
        System.out.println(applicationContext.getBeansOfType(SubjectMapper.class).toString());
        System.out.println(applicationContext.getBeansOfType(SubjectMapper.class).toString());
    }


    /**
     * Spring 条件注册
     */
    @Test
    public void testRegisterBeanByCondition() {
        ApplicationContext applicationContext = new AnnotationConfigApplicationContext(RegisterConditionConfig.class);
        Arrays.stream(applicationContext.getBeanDefinitionNames())
                .filter(beanName -> !beanName.contains("org.springframework"))
                .forEach(System.out::println);
    }


    /**
     * @code @Import 注册一个类
     */
    @Test
    public void testImportRegisterBean() {
        ApplicationContext applicationContext = new AnnotationConfigApplicationContext(ImportRegisterBeanConfiguration.class);
        Arrays.stream(applicationContext.getBeanDefinitionNames())
                .filter(beanName -> !beanName.contains("org.springframework"))
                .forEach(System.out::println);
    }


    /**
     * @code @Import + ImportSelector
     */
    @Test
    public void testImportBeanDefinitionRegistrar() {
        ApplicationContext applicationContext = new AnnotationConfigApplicationContext(BeanRegistrarConfig.class);
        Arrays.stream(applicationContext.getBeanDefinitionNames())
                .filter(beanName -> !beanName.contains("org.springframework"))
                .forEach(System.out::println);
    }

    /**
     * FactoryBean 注册一个 对象
     */
    @Test
    public void testFactoryBean() {
        ApplicationContext applicationContext = new AnnotationConfigApplicationContext(FactoryBeanConfig.class);
        // FactoryBean 生产 的Bean
        String beanName = "cn.dankal.share.factory.StudyFactoryBean";
        System.out.println("FactoryBean 生产 的Bean: \t" + applicationContext.getBean(beanName).getClass().getName());

        // FactoryBean 本身
        String realBeanName = "&cn.dankal.share.factory.StudyFactoryBean";
        System.out.println("FactoryBean 本身: \t" + applicationContext.getBean(realBeanName).getClass().getName());
    }

    /**
     * Spring Bean 的生命周期
     */
    @Test
    public void testBeanLifecycle() {
        // 容器初始化
        ConfigurableApplicationContext applicationContext = new AnnotationConfigApplicationContext(BeanLifecycleConfig.class);
        System.out.println("\nthis is java program\n");
        // 容器销毁
        applicationContext.close();
    }

    /**
     * @code BeanPostProcessor 后置处理器
     */
    @Test
    public void testBeanPostProcessor() {
        // 容器初始化
        ConfigurableApplicationContext applicationContext = new AnnotationConfigApplicationContext(BeanPostProcessorConfig.class);
        System.out.println("\nthis is java program\n");
        // 容器销毁
        applicationContext.close();
    }
}
